Unit CpuInfo;
{
 "Product of my Spare Time"

 !!! Some code (cpu speed) include from G. Bamber's Source. !!!

 UNIT: CpuInfo
 COMPONENT: TCPUFeatures
 LAST UPDATE: 11.07.98
 AUTHOR: L.Kovri (lkovari@freemail.c3.hu)
 DOCUMENTATION SOURCE: www.sandpile.org (cpuid)


 If you Found some bug refer me please...
 Thanks.
}


Interface

Uses
  SysUtils, Classes, Registry, Windows, Dialogs;

Const

  ID_BIT	                               = $200000;                    // EFLAGS ID bit


Type

  TVendor	 = Array [ 0..11 ] Of char;


  TCPUFeatures = Class ( TComponent )
  private
    fDummyInt : integer;
    fDummyStr : String;
    fDummydbl : double;
    fDummyComp : comp;
    fDummyBool : boolean;
    // Execution level need to get complet cache configuration
    fExecLevelNeed : integer;
    // Cpu Identification level
    fCpuidLevel : integer;
    // Cache configuration record EAX lo,hi EBX lo,hi ECX lo,hi EDX lo,hi
    fTLBInfoC,
      fTLBInfoD,
      fL1CacheC,
      fL1CacheD,
      fL2Cache : String;
    // 80386, 80486 Pentium
    fCpuType,
      // 12..13 bit primary, secondary, overdrive
    fCpuSubmodel,
      // 8..11 bit 486, 586, 686
    fCpuFamily,
      // 4..7 bit The processor model, like as iP54C
    fCpuModel,
      // 0..3 bit Stepping
    fCpuStep : String;
    // Intel, UMC, AMD, Cyrix, NexGen, CentaurHauls
    fCpuv : TVendor;
    fCpuVendor : String;
    //Persistent stuff
    fStartTime,
      fEndTime,
      fElapsed : TLargeInteger;
    fExactCpuSpeed : Double;
    fCpuSpeed : Comp;
    fCpuIdAvailable,
      // CPU Features fields, like as follows
    // 24. bit FXSAVE/FXRSTOR instruction are supported
    fCpuInstFxsaveSupport,
      // 23. bit MMX extension supported
    fCpuMMXSupport,
      // 17. bit 36bit Page Size Extension Supported
    fCpu36bitPageSizeExtSupport,
      // 16. bit Page Attribute Table is supported
    fCpuPageAttribTableSupport,
      // 15. bit CMOVcc (+FCMOVcc/F(U)COMI(P) if FPU=1 opcodes are supported
    fCpuCMOVccOpcodeSupport,
      // 14. bit Machine Check Architecture is supported
    fCpuMachineCheckSupport,
      // 13. bit Page Global Extension is supported
    fCpuPageGlobalExtensionSupport,
      // 12. bit Memory Type Range Registers are supported
    fCpuMTRRSuppoer,
      // 11. bit SYSENTER/SYSEXIT extension are supported
    fCpuSysenterExtSupport,
      // 9. bit processor contains an enabled APIC
    fCpuEnabledAPICContains,
      // 8. bit CMPXCHG8B instruction is supported
    fCpuCMPXCHG8BSupport,
      // 7. bit Machine Check Exception is supported
    fCpuMachineChkExceptionSupport,
      // 6. bit Physical Address Extension is supported
    fCpuPhysAddrExtSupport,
      // 5. bit Model Specific Registers are supported
    fCpuModelSpecRegSupport,
      // 4. bit Time Stamp Counter is supported
    fCpuTimeStampCntSupport,
      // 3. bit Page Size Extension is supported
    fCpuPageSizeExtSupport,
      // 2. bit Debugging Extension is supported
    fCpuDebuggingExtSupport,
      // 1. bit Virtual Mode Extension is supported
    fCpuVirtualModeExtSupport,
      // 0. bit processor contains a floating-point unit
    fCpuBuiltinFPU : boolean;
    Procedure DetCpuId;
    Procedure StartTimer;
    Procedure StopTimer;
    Procedure StoreCacheConfig;
    Function IntToHex ( N : longint ) : String;
    Function GetCpuType : String;
    Function GetCpuIdAvailable : Boolean;
    Function GetCpuIDLevel : integer;
    Function GetCPU_Vendor : String;
    Function GetCpuSpeedFromReg : Comp;
    Function GetTimeStamp : TLargeInteger;
    Function SupportsTimeStamp : Boolean;
    Function GetTicksPerSecond ( ITERATIONS : Word ) : Comp;
    Function GetExactCpuSpeed : Double;
    Function GetCpuSpeedWithOldMethode : Comp;
    Function GetCpuSpeed : Comp;
  protected
  public
    Constructor create ( AOwner : TComponent ); override;
    Destructor destroy; override;
  published
    Property CpuIdAvailable : Boolean read GetCpuIdAvailable write fDummyBool;
    Property CpuType : String read fCpuType write fDummyStr;
    Property CpuSubmodel : String read fCpuSubmodel write fDummyStr;
    Property CpuFamily : String read fCpuFamily write fDummyStr;
    Property CpuModel : String read fCpuModel write fDummyStr;
    Property CpuStepping : String read fCpuStep write fDummyStr;
    Property CpuVendor : String read fCpuVendor write fDummyStr;
    Property CpuSpeed : comp read fCpuSpeed write fDummyComp;
    Property CpuInstFxsaveSupport : boolean read fCpuInstFxsaveSupport write fDummyBool;
    Property CpuMMXSupport : boolean read fCpuMMXSupport write fDummyBool;
    Property Cpu36bitPageSizeExtSupport : boolean read fCpu36bitPageSizeExtSupport write
      fDummyBool;
    Property CpuPageAttribTableSupport : boolean read fCpuPageAttribTableSupport write
      fDummyBool;
    Property CpuCMOVccOpcodeSupport : Boolean read fCpuCMOVccOpcodeSupport write
      fDummyBool;
    Property CpuMachineCheckSupport : Boolean read fCpuMachineCheckSupport write
      fDummyBool;
    Property CpuPageGlobalExtensionSupport : Boolean read fCpuPageGlobalExtensionSupport
      write fDummyBool;
    Property CpuMTRRSuppoer : Boolean read fCpuMTRRSuppoer write fDummyBool;
    Property CpuSysenterExtSupport : Boolean read fCpuSysenterExtSupport write fDummyBool;
    Property CpuEnabledAPICContains : Boolean read fCpuEnabledAPICContains write
      fDummyBool;
    Property CpuCMPXCHG8BSupport : Boolean read fCpuCMPXCHG8BSupport write fDummyBool;
    Property CpuMachineChkExceptionSupport : Boolean read fCpuMachineChkExceptionSupport
      write fDummyBool;
    Property CpuPhysAddrExtSupport : Boolean read fCpuPhysAddrExtSupport write fDummyBool;
    Property CpuModelSpecRegSupport : Boolean read fCpuModelSpecRegSupport write
      fDummyBool;
    Property CpuTimeStampCntSupport : Boolean read fCpuTimeStampCntSupport write
      fDummyBool;
    Property CpuPageSizeExtSupport : Boolean read fCpuPageSizeExtSupport write fDummyBool;
    Property CpuDebuggingExtSupport : Boolean read fCpuDebuggingExtSupport write
      fDummyBool;
    Property CpuVirtualModeExtSupport : Boolean read fCpuVirtualModeExtSupport write
      fDummyBool;
    Property CpuBuiltinFPU : Boolean read fCpuBuiltinFPU write fDummyBool;
    Property CpuIdLevelSupport : integer read fCpuIdLevel write fDummyInt;
    Property CodeTLB : String read fTLBInfoC write fDummyStr;
    Property DataTLB : String read fTLBInfoD write fDummyStr;
    Property L1CodeCache : String read fL1CacheC write fDummyStr;
    Property L1DataCache : String read fL1CacheD write fDummyStr;
    Property L2Cache : String read fL2Cache write fDummyStr;
  End;




Var
  V                                     : String;
  Stepping, Family, Model, CPUTyp, Level : byte;
  Features                              : longint;
  Vn                                    : integer;
  TlbC,
    Tlbd,
    L1c,
    L1d,
    L2c,
    REaxlo,
    REaxhi,
    REbxlo,
    REbxhi,
    REcxlo,
		REcxhi,
		REdxlo,
		REdxhi                              : Word;


Procedure Register;

Implementation



Constructor TCPUFeatures.Create ( AOwner : TComponent );
Begin
	Inherited create ( AOwner );
  fCpuType := '';
  fCpuVendor := 'Not Detected';
  fCpuSubmodel := '';
  fCpuFamily := '';
  fCpuModel := '';
	fCpuStep := '';
  Try
    fCpuIdAvailable := GetCpuIdAvailable;
    If fCpuIdAvailable Then Begin
        fCpuIdLevel := GetCpuIdLevel;
        fCpuSpeed := GetCpuSpeed;
        fCpuType := GetCpuType;
        DetCpuId;
        Case CPUTyp Of
          3 : fCpuSubmodel := 'Reserved';
          2 : fCpuSubmodel := 'Secondary';
          1 : fCpuSubmodel := 'OverDrive';
          0 : fCpuSubmodel := 'Primary';
				Else
          fCpuSubmodel := 'Not Detected!';
        End;
        fCpuFamily := inttostr ( Family );
        fCpuModel := inttostr ( Model );
        fCpuStep := inttostr ( Stepping );
				V := GetCpu_Vendor;
        If v = 'GenuineIntel' Then
          Vn := 0
        Else
          If v = 'UMC UMC UMC' Then
          Vn := 1
        Else
          If v = 'AuthenticAMD' Then
          Vn := 2
        Else
          If v = 'CyrixInstead' Then
          Vn := 3
        Else
					If v = 'NexGenDriven' Then
          Vn := 4
        Else
          If v = 'CentaurHauls' Then
          Vn := 5;
        Case family Of
					4 :
            Case Vn Of
              0 :
                // Intel
                Case Model Of
                  0 : fCpuModel := 'i80486DX-25/33';
                  1 : fCpuModel := 'i80486DX-50';
                  2 : fCpuModel := 'i80486SX';
                  3 : fCpuModel := 'i80486DX2';
                  4 : fCpuModel := 'i80486SL';
                  5 : fCpuModel := 'i80486SX2';
                  7 : fCpuModel := 'i80486DX2WB';
                  8 : fCpuModel := 'i80486DX4';
									9 : fCpuModel := 'i80486DX4WB';
                End;
              1 :
                // United Microelectonics Corporation
                Case Model Of
                  1 : fCpuModel := 'U5D(486DX)';
									2 : fCpuModel := 'U5S(486SX)';
                End;
              2 :
                // AMD
                Case Model Of
                  3 : fCpuModel := '80486DX2WT';
                  7 : fCpuModel := '80486DX2WB';
                  8 : fCpuModel := '80486DX4';
                  9 : fCpuModel := '80486DX4WB';
                  14 : fCpuModel := '5x86';
                  15 : fCpuModel := '5x86WB';
                End;
              3 :
								// Cyrix
                Case Model Of
                  4 : fCpuModel := 'Cyrix Media GX';
                  9 : fCpuModel := 'Cyrix 5x86';
                End;
            End;

          5 :
            Case Vn Of
              0 :
                // Intel
                Case Model Of
                  0 : fCpuModel := 'P5 A-step';
                  1 : fCpuModel := 'P5';
                  2 : fCpuModel := 'P54C';
                  3 : fCpuModel := 'P24T OverDrive';
                  4 : fCpuModel := 'P55C';
                  5 : fCpuModel := 'DX4 OverDrive?';
                  6 : fCpuModel := 'P5 OverDrive?';
									7 : fCpuModel := 'P54C';
                  8 : fCpuModel := 'P55C(0,25m)MMX';
                End;
              2 :
                // AMD
                Case Model Of
									0 : fCpuModel := 'SSA5(75,90,100)';
                  1 : fCpuModel := '5k86(120,133)';
                  2 : fCpuModel := '5k86(166)';
                  3 : fCpuModel := '5k86(200)';
                End;
              3 :
                // Cyrix
                Case Model Of
                  0 : fCpuModel := 'Pentium Cx6X86 GXm';
                  2 : fCpuModel := 'Std. Cx6x86';
                  4 : fCpuModel := 'Cx6x86 GXm';
                End;
            Else
							// NexGenDriven
              If Vn = 4 Then
                fCpuModel := 'Nx586';
              // IDT/Centaur
              If Vn = 5 Then
                fCpuModel := 'IDT C6 (WinChip)';
						End;

          6 :
            Case Vn Of
              0 :
                // Intel
                Case Model Of
                  0 : fCpuModel := 'PentiumPro A-step';
                  1 : fCpuModel := 'PentiumPro';
                  3 : fCpuModel := 'Pentium II.';
                  4 : fCpuModel := 'P55CT (P54 overdrive)';
                  5 : fCpuModel := 'Pentium II.0,25m';
                End;
							2 :
                // AMD
                Case Model Of
                  6 : fCpuModel := 'K6(166,266)';
                  7 : fCpuModel := 'K6(166,300)';
                  8 : fCpuModel := 'K6-3D(200,450)';
									9 : fCpuModel := 'K6PLUS-3D(200,450)';
                End;
              3 :
                // Cyrix
                If Model = 0 Then
                  fCpuModel := 'Cx6x86 MX/MII';
            End;
        Else
          fCpuModel := 'Not Detected!';
        End;
        // Detect CPU Features
        If ( Features And $1000000 ) = $1000000 Then
          fCpuInstFxsaveSupport := True;
				If ( Features And $800000 ) = $800000 Then
          fCpuMMXSupport := True;
        If ( Features And $20000 ) = $20000 Then
          fCpu36bitPageSizeExtSupport := True;
        If ( Features And $10000 ) = $10000 Then
          fCpuPageAttribTableSupport := True;
				If ( Features And $8000 ) = $8000 Then
          fCpuCMOVccOpcodeSupport := True;
        If ( Features And $4000 ) = $4000 Then
          fCpuMachineCheckSupport := True;
        If ( Features And $2000 ) = $2000 Then
          fCpuPageGlobalExtensionSupport := True;
        If ( Features And $1000 ) = $1000 Then
          fCpuMTRRSuppoer := True;
        If ( Features And $800 ) = $800 Then
          fCpuSysenterExtSupport := True;
        If ( Features And $200 ) = $200 Then
          fCpuEnabledAPICContains := True;
        If ( Features And $100 ) = $100 Then
					fCpuCMPXCHG8BSupport := True;
        If ( Features And $80 ) = $80 Then
          fCpuMachineChkExceptionSupport := True;
        If ( Features And $40 ) = $40 Then
          fCpuPhysAddrExtSupport := True;
        If ( Features And $20 ) = $20 Then
					fCpuModelSpecRegSupport := True;
        If ( Features And $10 ) = $10 Then
          fCpuTimeStampCntSupport := True;
        If ( Features And 8 ) = 8 Then
          fCpuPageSizeExtSupport := True;
        If ( Features And 4 ) = 4 Then
          fCpuDebuggingExtSupport := True;
        If ( Features And 2 ) = 2 Then
          fCpuVirtualModeExtSupport := True;
        If ( Features And 1 ) = 1 Then
          fCpuBuiltinFPU := True;
        If fCpuIdLevel > 1 Then Begin
            // Store Cache info to the Variables
						StoreCacheConfig;
            //  Descriptor validation check
            If ( REaxhi And $1000 ) = 0 Then Begin
                fExecLevelNeed := ( REaxlo And $00FF );
                //Tlb:=Reaxhi and $ff00;
                Case Tlbd Of
									$03 : fTLBInfoD := 'Data TLB has 64 Entries, 4K Pages, 4 Ways';
                  $04 : fTLBInfoD := 'Data TLB has 8 Entries, 4M Pages, 4 Ways';
                End;
                //Tlb:=REaxhi and $00ff;
                Case Tlbc Of
                  $01 : fTLBInfoC := 'Code TLB has 32 Entries, 4K Pages, 4 Ways';
                  $02 : fTLBInfoC := 'Code TLB has 2 Entries, 4M Pages, Fully';
                End;
              End;
            If ( REdxhi And $1000 ) = 0 Then Begin
                //L1:=REdxhi and $ff00;
                Case L1d Of
                  $0A : fL1CacheD := 'Data L1 Cache has 32 byte lines, 8Kb, 2 Ways';
									$0C : fL1CacheD := 'Data L1 Cache has 32 byte lines, 16Kb, 4 Ways';
                End;
                //L1:=REdxlo and $ff00;
                Case L1c Of
                  $06 : fL1CacheC := 'Code L1 Cache has 32 byte lines, 8Kb, 4 Ways';
                  $08 : fL1CacheC := 'Code L1 Cache has 32 byte lines, 16Kb, 4 Ways';
								End;
                //L2:=REdxlo and $00ff;
                Case L2c Of
                  $40 : fL2Cache := 'No Integrated L2 Cache';
                  $41 : fL2Cache := 'Code & Data L2 Cache, 128Kb, 4 Ways, 32 byte lines';
                  $42 : fL2Cache := 'Code & Data L2 Cache, 256Kb, 4 Ways, 32 byte lines';
                  $43 : fL2Cache := 'Code & Data L2 Cache, 512Kb, 4 Ways, 32 byte lines';
                  $44 : fL2Cache := 'Code & Data L2 Cache, 1024Kb, 4 Ways, 32 byte lines';
                  $45 : fL2Cache := 'Code & Data L2 Cache, 2048Kb, 4 Ways, 32 byte lines';
                  $70 :
                    fL2Cache :=
                      'Cyrix Specific: Code & Data TLB 32 Entries, 4K Pages, 4 Ways';
                  $80 :
										fL2Cache :=
                      'Cyrix Specific: Code & Data L1 Cache 16 byte lines, 16K Pages, 4 Ways';
                End;
              End;
          End;
      End;
	Except
		On E : Exception Do begin
			MessageDlg ( 'Error Create the TCPUFeatures Component in CpuInfo Unit' + #13 +
				E.Message, mtError, [ mbOK ], 0 );
			Exit;
		end;
	End;
End;

Procedure TCpuFeatures.StoreCacheConfig;
Begin

	Asm
		mov eax, 2			//  Get Cache Configuration
		db 0Fh,0a2h     //	CPUID opcode

		push	eax
		mov	REaxlo,ax
		shr	eax,16
		mov	REaxhi,ax
		pop	eax

		push	eax
		and	Eax,0ff000000h
		shr	eax,24
		mov	Tlbd,ax

		pop	eax
		and	Eax,00ff0000h
		shr	eax,16
		mov	Tlbc,ax

		push	edx
		mov	REdxlo,dx
		shr	edx,16
		mov	REdxhi,dx
		pop	edx

		push	edx
		and	edx,0ff000000h
		shr	edx,24
		mov	L1d,dx
		pop	edx

		push	edx
		and	edx,0000ff00h
		shr	edx,8
		mov	L1c,dx

		pop	edx
		and edx,000000ffh
		mov	L2c,dx

		ret
  End;
End;

Destructor TCPUFeatures.Destroy;
Begin
  Inherited destroy;

End;

Procedure Register;
Begin
  RegisterComponents ( 'Freeware', [ TCpuFeatures ] );
End;

Function TCPUFeatures.GetCpuType : String;


  Function GetType : integer; assembler;
  Asm
	push EBX
	push ECX
	push EDX

	mov ebx,esp
	and esp,0FFFFFFFCh  //align down to nearest dword
	pushfd              //save original flags

	// i386 CPU check
	//	The AC bit, bit #18, is a new bit introduced in the EFLAGS
	//	register on the i486 DX CPU to generate alignment faults.
	//	This bit can not be set on the i386 CPU.

	pushfd
	pop eax
	mov ecx,eax
	xor eax,40000h   	//toggle AC bit
	push eax
	popfd
	pushfd
	pop eax
	xor eax,ecx
	mov eax,3          //assume 80386
	je @@End_CpuType   //it's a 386

	// i486 DX CPU / i487 SX MCP and i486 SX CPU checking
	// Checking for ability to set/clear ID flag (Bit 21) in EFLAGS
	// which indicates the presence of a processor
	// with the ability to use the CPUID instruction.

	pushfd
	pop	eax
	mov	ecx,eax
	xor	eax,200000h   //toggle ID bit
	push eax
	popfd
	pushfd
	pop eax
	xor eax, ecx
	mov eax,4
	je @@End_CpuType   //it's a 486 w/o support for CPUID

		// 	Execute CPUID instruction to determine vendor, family,
		//		model and stepping.  The use of the CPUID instruction used
		//		in this program can be used for B0 and later steppings
		//		of the P5 processor.

	push ebx        // 	CPUID modifies EBX  !!!
	mov eax, 1			//   Level
	db 0Fh,0a2h     //	CPUID opcode
	mov al,ah
	and eax, 0FH
	pop ebx

@@End_CpuType:
	popfd            // restore original flags
	mov esp,ebx      // restore original ESP
	pop EDX
	pop ECX
	pop EBX
  End;


Begin
  Result := 'Not Detected!';
  If GetCpuIdAvailable Then
    Case GetType Of
      3 : Result := '80386';
      4 : Result := '80486';
      5 : Result := 'Pentium';
      6 : Result := '6x86 Family';
    Else
      If GetType > 6 Then
        Result := 'Unknown Family';
    End;
End;


Function TCPUFeatures.GetCpuIdLevel : integer;
Begin
  Level := 0;
  Asm
		mov eax, 0			//  Get Level
		db 0Fh,0a2h     //	CPUID opcode
		mov Level,al
		ret
  End;
  result := Level;
End;


Function TCPUFeatures.GetCpuIdAvailable : Boolean; register;
Asm
	PUSHFD							//direct access to flags no possible, only via stack
	POP     EAX					//flags to EAX
	MOV     EDX,EAX			//save current flags
	XOR     EAX,ID_BIT	//not ID bit
	PUSH    EAX					//onto stack
	POPFD								//from stack to flags, with not ID bit
	PUSHFD							//back to stack
	POP     EAX					//get back to EAX
	XOR     EAX,EDX			//check if ID bit affected
	JZ      @exit				//no, CPUID not availavle
	MOV     AL,True			//Result=True
@exit:
End;

Procedure TCPUFeatures.DetCpuId; assembler;
Asm
	PUSH    EAX               //Save reg.
	PUSH    EBX
	PUSH    EDX
	PUSH    ECX
	MOV     EAX,1
	DW      $A20F             //CPUID Command Execute
	push    eax
	mov     Stepping,al       //Store Stepping
	and     Stepping,0fh      //Stepping mask
	and     al,0f0h           //Model mask
	shr     al,4              //Model shift
	mov     Model,al          //Store Model
	and     ax,0f00h          //Family mask
	shr     ax,8              //Family shift
	mov     Family,al         //Store Family
	and     al,0f0h;          //Type mask
	shr     al,4              //type shift
	mov     CpuTyp,al         //Store Type
	pop     eax
	mov     Features,edx      //Store features
	POP     ECX
	POP     EDX
	POP     EBX
	POP     EAX               //Restore Reg.
End;


Function TCPUFeatures.GetCPU_Vendor : String;
Var
  i                                     : byte;
  s                                     : String;

  Function GetCPUVendor : TVendor; assembler; register;
  Asm
	PUSH    EBX				//Save regs
	PUSH    EDI
	MOV     EDI,EAX		//@Result (TVendor)
	MOV     EAX,0
	DW      $A20F			//CPUID Command
	MOV     EAX,EBX
	XCHG	  EBX,ECX   //save ECX result
	MOV	  ECX,4
@1:
	STOSB            	//save first 4 byte
	SHR     EAX,8
	LOOP    @1
	MOV     EAX,EDX
	MOV	  ECX,4
@2:
	STOSB            	//save middle 4 byte
	SHR     EAX,8
	LOOP    @2
	MOV     EAX,EBX
	MOV	  ECX,4
@3:            			//save last 4 byte
	STOSB
	SHR     EAX,8
	LOOP    @3
	POP     EDI				//Restore regs
	POP     EBX
  End;

Begin
  fCpuV := GetCpuVendor;
  fCpuVendor := '';
  i := 1;
  s := '';
  Repeat
    s := fCpuV [ i - 1 ];
    fCpuVendor := fCpuVendor + s;
    i := i + 1;
  Until i > 12;
  Result := fCpuVendor;
End;


Function TCPUFeatures.GetCpuSpeedFromReg : Comp;
Var
  MyReg                                 : TRegistry;
  B                                     : LongInt;
  Mhz                                   : LongInt;
Begin
  Result := 0;
  MHz := 0;
  Try
    MyReg := TRegistry.Create;
    Try
      MyReg.RootKey := HKEY_LOCAL_MACHINE;
      If MyReg.OpenKey ( '\HARDWARE\DESCRIPTION\System\CentralProcessor\0', FALSE ) Then
        Begin
          B := MyReg.ReadInteger ( '~MHz' );
          If B > 0 Then
            Mhz := B
          Else
            B := MyReg.ReadInteger ( '~Mhz' );
          If B > 0 Then
            Mhz := B
          Else
            B := MyReg.ReadInteger ( '~mhz' );
          If B > 0 Then
            Mhz := B;
        End;
    Finally
      MyReg.Free;
    End;
  Except
    Result := 0;
  End;
  //Convert MegaHertz to Hertz
  Result := Mhz * 1E6;
  fCPUSpeed := Result;
End;

Function GetCPUID4 : DWORD; assembler; register;
// This function retrieves a set of CPUID flags,
// one of which, is whether the TimeStamp counter exists
Asm
	PUSH    EBX			//Save registers
	PUSH    EDI
	MOV     EAX,1 	//Set up for CPUID
	DW      $A20F 	//CPUID OpCode
	MOV @Result,EDX	//Put the flag array into a DWord
	POP     EDI			//Restore registers
	POP     EBX
End;

// Not Used in this Component (Included for the Future)

Function TCpuFeatures.SupportsTimeStamp : Boolean;
Begin
  Result := FALSE;
  If ( GetCPUID4 And 16 ) = 16 Then
    result := TRUE;
End;

Function GetTimeStampHi : DWord; assembler; register;
Asm
 DW      $310F       //RDTSC Command
 MOV @Result, EDX;
End;


Function GetTimeStampLo : DWord; assembler; register;
Asm
	DW      $310F       //RDTSC Command
	MOV @Result, EAX;
End;


Function TCpuFeatures.GetTimeStamp : TLargeInteger;
Begin
  Result.QuadPart := 0;
  If ( GetCPUID4 And 16 ) <> 16 Then exit; //No Timestamp Function
  Result.HighPart := DWORD ( GetTimeStampHi );
  Result.LowPart := GetTimeStampLo;
End;


Procedure TCpuFeatures.StartTimer;
Begin
  //Get raw processor ticks since power-on
  fStartTime := GetTimeStamp;
  fEndTime.QuadPart := 0;
  fElapsed.QuadPart := 0;
End;


Procedure TCpuFeatures.StopTimer;
Begin
  //Get raw processor ticks since power-on
  fEndTime := GetTimeStamp;
  fElapsed.QuadPart := ( fEndTime.QuadPart - fStartTime.QuadPart );
End;


Function TCpuFeatures.GetTicksPerSecond ( ITERATIONS : Word ) : Comp;
Var
  FREQ, PERFCOUNT, TARGET               : TLARGEINTEGER;
Begin
  Result := 0; //Initialise in case of failure
  If Not QueryPerformanceFrequency ( FREQ ) Then exit;
  //The OS or hardware may not support this API call.
  //Sometimes, the FREQ is equal to Processor Ticks/Second,
  //but it might not be on every system

  QueryPerformanceCounter ( PERFCOUNT );
  //Get a start time via the OS
  TARGET.QuadPart := PERFCOUNT.QuadPart + ( FREQ.QuadPart * ITERATIONS );
  //Make an end time
  StartTimer;
  //Fetch the processor tickcount
  Repeat // Loop for exactly one second (according to the OS)
    QueryPerformanceCounter ( PERFCOUNT );
  Until ( PERFCOUNT.QuadPart >= TARGET.QuadPart );
  //Fetch the number of processor ticks since the loop started
  StopTimer;
  Result := ( fElapsed.QuadPart / ITERATIONS );
  //In MegaHertz
  fExactCPUSpeed := Result / 1E6;
End;


Function TCpuFeatures.GetExactCpuSpeed : Double;
Var
  MYCOMP                                : Comp;
Begin
  //Do a 10-second test
  MYCOMP := GetTicksPerSecond ( 1 );
  Result := MYCOMP / 1E6;

  //In MegaHertz
  fExactCPUSpeed := Result;
End;

Function TCpuFeatures.GetCpuSpeedWithOldMethode : Comp;
// Uses RDTSC Command to get the Processor Timer count
Var
  t                                     : DWORD;
  mhi, mlo, nhi, nlo                    : DWORD;
  t0, t1, chi, clo, shr32               : Comp;
Begin
  Result := 0;
  shr32 := 65536;
  shr32 := shr32 * 65536;
  // Get a start time
  t := GetTickCount;
  While t = GetTickCount Do Begin
    End;
  Asm
		DB 0FH
		DB 031H  		// RDTSC Opcode retrieves 64-bit tick count since boot
		mov mhi,edx // 32-bit Ticks Hi
		mov mlo,eax // 32-bit Ticks Lo
  End;
  // Look 1 second later
  While GetTickCount < ( t + 1000 ) Do Begin
    End;
  Asm
		DB 0FH
		DB 031H  		// RDTSC
		mov nhi,edx
		mov nlo,eax
  End;
  // See how many Processor Timer Ticks have elapsed
  chi := mhi;
  If mhi < 0 Then
    chi := chi + shr32;
  clo := mlo;
  If mlo < 0 Then
    clo := clo + shr32;
  t0 := chi * shr32 + clo;
  chi := nhi;
  If nhi < 0 Then
    chi := chi + shr32;
  clo := nlo;
  If nlo < 0 Then
    clo := clo + shr32;
  t1 := chi * shr32 + clo;
  Result := ( t1 - t0 );
End;

// Not Used in this Component (Maked for the Future)

Function TCpuFeatures.inttoHex ( N : longint ) : String;
Const
  hexChars                              : Array [ 0..$F ] Of Char = '0123456789ABCDEF';
Var
  i                                     : byte;
  v                                     : longint;
  s                                     : String;
Begin
  s := '                                ';
  v := N;
  i := 32;
  Repeat
    s [ i ] := hexChars [ v Mod 16 ];
    v := trunc ( v / 16 );
    i := i - 1;
  Until v <= 0;
  result := trim ( s );
End;


Function TCpuFeatures.GetCpuSpeed : comp;
Begin
  If fCpuSpeed = 0 Then
    fCpuSpeed := GetCpuSpeedFromReg / 1E6;
  If fCpuSpeed = 0 Then
    fCpuSpeed := GetCpuSpeedWithOldMethode / 1E6;
  If fCpuSpeed = 0 Then
    fCpuSpeed := GetExactCpuSpeed;
  Result := fCpuSpeed;
End;



End.

